@@ -26,6 +26,9 @@
 #include "layoutwidgetfield.h"
 #include <libglademm.h>
 
+namespace Glom
+{
+
 class App_Glom;
 
 class ImageGlom
@@ -50,13 +53,17 @@ public:
 
   void set_read_only(bool read_only = true);
 
+  static Glib::RefPtr<Gdk::Pixbuf> scale_keeping_ratio(const Glib::RefPtr<Gdk::Pixbuf>& pixbuf, int target_height, int target_width);
+
 protected:
   void init();
 
-
+  // Note that these are normal signal handlers when glibmm was compiled
+  // without default signal handler API.
   virtual bool on_expose_event(GdkEventExpose* event);
 
   virtual bool on_button_press_event(GdkEventButton *event);
+
   void on_menupopup_activate_select_file();
   void on_menupopup_activate_copy();
   void on_menupopup_activate_paste();
@@ -71,8 +78,7 @@ protected:
   void setup_menu_usermode();
   void scale();
 
-  static Glib::RefPtr<Gdk::Pixbuf> scale_keeping_ratio(const Glib::RefPtr<Gdk::Pixbuf>& pixbuf, int target_height, int target_width);
-
+ 
   Gtk::Image m_image;
   Gtk::Frame m_frame;
   Glib::RefPtr<Gdk::Pixbuf> m_pixbuf_original; //Only stored temporarily, because it could be big.
@@ -86,5 +92,7 @@ protected:
   bool m_read_only;
 };
 
+} //namespace Glom
+
 #endif //GLOM_UTILITY_WIDGETS_COMBOENTRY_GLOM_H
 
